//
use actix::prelude::*;

use tracing::{info, Instrument as _};

use crate::alerts::{
    desktop::send_desktop_notification, email::send_email, messages::NotifyUser,
    webhook::send_webhook, AlertsActor,
};

impl Handler<NotifyUser> for AlertsActor {
    type Result = ();

    fn handle(&mut self, msg: NotifyUser, ctx: &mut Self::Context) -> Self::Result {
        let Some(shout) = &self.shout else {
            info!("No shout config available");
            return;
        };
        let net = self.net.clone();
        let shout = shout.clone();
        let Some(user_notification) = self.history.sendable(msg.peel()) else {
            return;
        };
        async move {
            if let Some(webhook_config) = shout.webhook() {
                send_webhook(&user_notification, webhook_config, &net).await;
            }
            if let Some(email_config) = shout.email() {
                send_email(&user_notification, email_config);
            }
            if let Some(desktop) = shout.desktop() {
                if desktop {
                    send_desktop_notification(&user_notification);
                }
            }
        }
        .in_current_span()
        .into_actor(self)
        .wait(ctx);
    }
}
