//
use actix::prelude::*;

use tracing::{debug, Instrument as _};

use crate::{
    repo::{
        do_send,
        messages::{CheckCIStatus, ReceiveCIStatus},
        RepoActor,
    },
    server::actor::messages::RepoUpdate,
};

impl Handler<CheckCIStatus> for RepoActor {
    type Result = ();

    fn handle(&mut self, msg: CheckCIStatus, ctx: &mut Self::Context) -> Self::Result {
        crate::repo::logger(self.log.as_ref(), "start: CheckCIStatus");

        let addr = ctx.address();
        let forge = self.forge.duplicate();
        let next = msg.peel();
        let log = self.log.clone();

        self.update_tui(RepoUpdate::CheckingCI);
        // get the status - pass, fail, pending (all others map to fail, e.g. error)
        async move {
            let status = forge.commit_status(&next).await;
            debug!("got status: {status:?}");
            do_send(&addr, ReceiveCIStatus::new((next, status)), log.as_ref());
        }
        .in_current_span()
        .into_actor(self)
        .wait(ctx);
    }
}
