//
use actix::prelude::*;

use git_next_core::git;
use tracing::{debug, instrument, warn};

use crate::{
    repo::{
        do_send, logger,
        messages::{CloneRepo, LoadConfigFromRepo, RegisterWebhook},
        RepoActor,
    },
    server::actor::messages::RepoUpdate,
};

impl Handler<CloneRepo> for RepoActor {
    type Result = ();
    #[instrument(name = "RepoActor::CloneRepo", skip_all, fields(repo = %self.repo_details))]
    fn handle(&mut self, _msg: CloneRepo, ctx: &mut Self::Context) -> Self::Result {
        logger(self.log.as_ref(), "Handler: CloneRepo: start");
        self.update_tui(RepoUpdate::Opening);
        debug!("Handler: CloneRepo: start");
        match git::repository::open(&*self.repository_factory, &self.repo_details) {
            Ok(repository) => {
                logger(self.log.as_ref(), "open okay");
                debug!("open okay");
                self.update_tui(RepoUpdate::Opened);
                self.open_repository.replace(repository);
                if self.repo_details.repo_config.is_none() {
                    do_send(&ctx.address(), LoadConfigFromRepo, self.log.as_ref());
                } else {
                    do_send(&ctx.address(), RegisterWebhook::new(), self.log.as_ref());
                }
            }
            Err(err) => {
                logger(self.log.as_ref(), "open failed");
                warn!("Could not open repo: {err:?}");
                self.alert_tui(err.to_string());
            }
        }
        debug!("Handler: CloneRepo: finish");
    }
}
