//
use actix::prelude::*;

use git_next_core::git::UserNotification;

use tracing::{debug, instrument, Instrument as _};

use crate::{
    repo::{
        do_send, load,
        messages::{LoadConfigFromRepo, ReceiveRepoConfig},
        notify_user, RepoActor,
    },
    server::actor::messages::RepoUpdate,
};

impl Handler<LoadConfigFromRepo> for RepoActor {
    type Result = ();
    #[instrument(name = "Repocrate::repo::LoadConfigFromRepo", skip_all, fields(repo = %self.repo_details))]
    fn handle(&mut self, _msg: LoadConfigFromRepo, ctx: &mut Self::Context) -> Self::Result {
        debug!("Handler: LoadConfigFromRepo: start");
        self.update_tui(RepoUpdate::LoadingConfigFromRepo);
        let Some(open_repository) = &self.open_repository else {
            return;
        };
        let open_repository = open_repository.duplicate();
        let repo_details = self.repo_details.clone();
        let forge_alias = repo_details.forge.forge_alias().clone();
        let repo_alias = repo_details.repo_alias.clone();
        let addr = ctx.address();
        let notify_user_recipient = self.notify_user_recipient.clone();
        let log = self.log.clone();
        async move {
            match load::config_from_repository(repo_details, &*open_repository).await {
                Ok(repo_config) => {
                    do_send(&addr, ReceiveRepoConfig::new(repo_config), log.as_ref());
                }
                Err(err) => notify_user(
                    notify_user_recipient.as_ref(),
                    UserNotification::RepoConfigLoadFailure {
                        forge_alias,
                        repo_alias,
                        reason: err.to_string(),
                    },
                    log.as_ref(),
                ),
            }
        }
        .in_current_span()
        .into_actor(self)
        .wait(ctx);
        debug!("Handler: LoadConfigFromRepo: finish");
    }
}
