//
use actix::prelude::*;
use tracing::instrument;

use crate::{
    repo::{
        do_send,
        messages::{ReceiveRepoConfig, RegisterWebhook},
        RepoActor,
    },
    server::actor::messages::RepoUpdate,
};

impl Handler<ReceiveRepoConfig> for RepoActor {
    type Result = ();
    #[instrument(name = "RepoActor::ReceiveRepoConfig", skip_all, fields(repo = %self.repo_details, branches = ?msg))]
    fn handle(&mut self, msg: ReceiveRepoConfig, ctx: &mut Self::Context) -> Self::Result {
        let repo_config = msg.peel();
        self.update_tui(RepoUpdate::ReceiveRepoConfig {
            repo_config: repo_config.clone(),
        });
        self.repo_details.repo_config.replace(repo_config);
        self.update_tui_branches();
        do_send(&ctx.address(), RegisterWebhook::new(), self.log.as_ref());
    }
}
