//
use actix::prelude::*;

use tracing::{debug, error, Instrument as _};

use crate::{
    repo::{
        do_send,
        messages::{RegisterWebhook, WebhookRegistered},
        notify_user, RepoActor,
    },
    server::actor::messages::RepoUpdate,
};

use git_next_core::git::UserNotification;

impl Handler<RegisterWebhook> for RepoActor {
    type Result = ();

    fn handle(&mut self, _msg: RegisterWebhook, ctx: &mut Self::Context) -> Self::Result {
        if self.webhook_id.is_none() {
            let forge_alias = self.repo_details.forge.forge_alias().clone();
            let repo_alias = self.repo_details.repo_alias.clone();
            let repo_listen_url = self
                .listen_url
                .repo_url(forge_alias.clone(), repo_alias.clone());
            let forge = self.forge.duplicate();
            let addr = ctx.address();
            let notify_user_recipient = self.notify_user_recipient.clone();
            let log = self.log.clone();
            self.update_tui(RepoUpdate::RegisteringWebhook);
            debug!("registering webhook");
            async move {
                match forge.register_webhook(&repo_listen_url).await {
                    Ok(registered_webhook) => {
                        debug!(?registered_webhook, "webhook registered");
                        do_send(
                            &addr,
                            WebhookRegistered::from(registered_webhook),
                            log.as_ref(),
                        );
                    }
                    Err(err) => {
                        error!(?err, "failed to register webhook");
                        notify_user(
                            notify_user_recipient.as_ref(),
                            UserNotification::WebhookRegistration {
                                forge_alias,
                                repo_alias,
                                reason: err.to_string(),
                            },
                            log.as_ref(),
                        );
                    }
                }
            }
            .in_current_span()
            .into_actor(self)
            .wait(ctx);
        } else {
            self.alert_tui("already have a webhook id - cant register webhook");
        }
    }
}
