//
use actix::prelude::*;

use tracing::{debug, warn, Instrument as _};

use crate::{
    repo::{messages::UnRegisterWebhook, RepoActor},
    server::actor::messages::RepoUpdate,
};

impl Handler<UnRegisterWebhook> for RepoActor {
    type Result = ();

    fn handle(&mut self, _msg: UnRegisterWebhook, ctx: &mut Self::Context) -> Self::Result {
        let Some(webhook_id) = self.webhook_id.take() else {
            return;
        };
        self.update_tui(RepoUpdate::UnregisteringWebhook);
        let forge = self.forge.duplicate();
        debug!("unregistering webhook");
        async move {
            match forge.unregister_webhook(&webhook_id).await {
                Ok(()) => debug!("unregistered webhook"),
                Err(err) => warn!(?err, "unregistering webhook"),
            }
        }
        .in_current_span()
        .into_actor(self)
        .wait(ctx);
        debug!("unregistering webhook done");
    }
}
