//
use actix::prelude::*;
use tracing::instrument;

use crate::{
    repo::{
        do_send,
        messages::{ValidateRepo, WebhookRegistered},
        RepoActor,
    },
    server::actor::messages::RepoUpdate,
};

impl Handler<WebhookRegistered> for RepoActor {
    type Result = ();
    #[instrument(name = "RepoActor::WebhookRegistered", skip_all, fields(repo = %self.repo_details, webhook_id = %msg.webhook_id()))]
    fn handle(&mut self, msg: WebhookRegistered, ctx: &mut Self::Context) -> Self::Result {
        self.update_tui(RepoUpdate::RegisteredWebhook);
        self.webhook_id.replace(msg.webhook_id().clone());
        self.webhook_auth.replace(msg.webhook_auth().clone());
        do_send(
            &ctx.address(),
            ValidateRepo::new(self.message_token),
            self.log.as_ref(),
        );
    }
}
