//
use super::*;

#[actix::test]
async fn should_passthrough_to_receive_ci_status() -> TestResult {
    //given
    let fs = given::a_filesystem();
    let (open_repository, repo_details) = given::an_open_repository(&fs);
    let next_commit = given::a_named_commit("next");
    let mut forge = git::MockForgeLike::new();
    when::commit_status(
        &mut forge,
        next_commit.clone(),
        git::forge::commit::Status::Pass,
    );

    //when
    let (addr, log) = when::start_actor_with_open_repository(
        Box::new(open_repository),
        repo_details,
        Box::new(forge),
    );
    addr.send(crate::repo::messages::CheckCIStatus::new(
        next_commit.clone(),
    ))
    .await?;
    System::current().stop();

    //then
    tracing::debug!(?log, "");
    log.read().map_err(|e| e.to_string()).map(|l| {
        assert!(l
            .iter()
            .any(|message| message.contains("send: ReceiveCIStatus")));
    })?;
    Ok(())
}
