use actix::prelude::*;

use crate::server::actor::{
    messages::{ReceiveAppConfig, ReceiveValidAppConfig, ValidAppConfig},
    ServerActor,
};

impl Handler<ReceiveAppConfig> for ServerActor {
    type Result = ();

    #[allow(clippy::cognitive_complexity)]
    fn handle(&mut self, msg: ReceiveAppConfig, ctx: &mut Self::Context) -> Self::Result {
        tracing::info!("recieved server config");
        let Ok(socket_addr) = msg.listen_socket_addr() else {
            return self.abort(ctx, "Unable to parse http.addr");
        };

        let Some(server_storage) = self.server_storage(&msg) else {
            return self.abort(ctx, "Server storage not available");
        };

        if msg.listen().url().ends_with('/') {
            return self.abort(ctx, "webhook.url must not end with a '/'");
        }

        self.do_send(
            ReceiveValidAppConfig::new(ValidAppConfig::new(
                msg.peel(),
                socket_addr,
                server_storage,
            )),
            ctx,
        );
    }
}
