//-

use actix::prelude::*;
use tracing::debug;

use crate::{
    repo::messages::UnRegisterWebhook,
    server::actor::{messages::Shutdown, ServerActor},
    webhook::messages::ShutdownWebhook,
};

impl Handler<Shutdown> for ServerActor {
    type Result = ();

    fn handle(&mut self, _msg: Shutdown, _ctx: &mut Self::Context) -> Self::Result {
        self.repo_actors
            .iter()
            .for_each(|((forge_alias, repo_alias), addr)| {
                debug!(%forge_alias, %repo_alias, "removing webhook");
                addr.do_send(UnRegisterWebhook::new());
                debug!(%forge_alias, %repo_alias, "removed webhook");
            });
        debug!("server shutdown");
        if let Some(webhook) = self.webhook_actor_addr.take() {
            debug!("shutting down webhook");
            webhook.do_send(ShutdownWebhook);
            debug!("webhook shutdown");
        }
    }
}
