//
use std::collections::BTreeMap;

use git_next_core::{ForgeAlias, RepoAlias};
use ratatui::{
    buffer::Buffer,
    layout::{Alignment, Direction, Layout, Rect},
    widgets::{block::Title, Block, Widget},
};

use crate::tui::{
    actor::RepoState,
    components::{repo::RepoWidget, HeightContraintLength},
};

pub struct ExpandedForgeWidget<'a> {
    pub forge_alias: &'a ForgeAlias,
    pub repos: &'a BTreeMap<RepoAlias, RepoState>,
}
impl<'a> HeightContraintLength for ExpandedForgeWidget<'a> {
    fn height_constraint_length(&self) -> u16 {
        self.children()
            .iter()
            .map(HeightContraintLength::height_constraint_length)
            .sum::<u16>()
            + 2 // top title + bottom padding
    }
}
impl<'a> Widget for ExpandedForgeWidget<'a> {
    fn render(self, area: Rect, buf: &mut Buffer)
    where
        Self: Sized,
    {
        let block = Block::default().title(
            Title::from(format!(" forge: {} ", self.forge_alias)).alignment(Alignment::Left),
        );
        let children = self.children();
        let layout = Layout::default()
            .direction(Direction::Vertical)
            .constraints(
                children
                    .iter()
                    .map(HeightContraintLength::height_constraint_length),
            )
            .split(block.inner(area));
        block.render(area, buf);
        children
            .into_iter()
            .enumerate()
            .for_each(|(i, w)| w.render(layout[i], buf));
    }
}
impl<'a> ExpandedForgeWidget<'a> {
    fn children(&self) -> Vec<RepoWidget<'a>> {
        self.repos
            .values()
            .map(|repo_state| RepoWidget { repo_state })
            .collect::<Vec<_>>()
    }
}
