//
mod collapsed;
mod expanded;

use std::collections::BTreeMap;

use collapsed::CollapsedForgeWidget;
use expanded::ExpandedForgeWidget;
use git_next_core::{ForgeAlias, RepoAlias};
use ratatui::{buffer::Buffer, layout::Rect, widgets::Widget};

use crate::tui::actor::{RepoState, ViewState};

use super::HeightContraintLength;

pub struct ForgeWidget<'a> {
    pub forge_alias: &'a ForgeAlias,
    pub repos: &'a BTreeMap<RepoAlias, RepoState>,
    pub view_state: ViewState,
}
impl<'a> HeightContraintLength for ForgeWidget<'a> {
    fn height_constraint_length(&self) -> u16 {
        match self.view_state {
            ViewState::Collapsed => CollapsedForgeWidget {
                forge_alias: self.forge_alias,
            }
            .height_constraint_length(),
            ViewState::Expanded => ExpandedForgeWidget {
                forge_alias: self.forge_alias,
                repos: self.repos,
            }
            .height_constraint_length(),
        }
    }
}
impl<'a> Widget for ForgeWidget<'a> {
    fn render(self, area: Rect, buf: &mut Buffer)
    where
        Self: Sized,
    {
        match self.view_state {
            ViewState::Collapsed => CollapsedForgeWidget {
                forge_alias: self.forge_alias,
            }
            .render(area, buf),
            ViewState::Expanded => ExpandedForgeWidget {
                forge_alias: self.forge_alias,
                repos: self.repos,
            }
            .render(area, buf),
        }
    }
}
