//
use actix::prelude::*;

mod handlers;
pub mod messages;
pub mod router;
mod server;

use crate::repo::messages::WebhookNotification;

use std::net::SocketAddr;

use tracing::Instrument;

#[allow(clippy::module_name_repetitions)]
#[derive(Debug)]
pub struct WebhookActor {
    socket_addr: SocketAddr,
    span: tracing::Span,
    spawn_handle: Option<actix::SpawnHandle>,
    message_receiver: Recipient<WebhookNotification>,
}
impl WebhookActor {
    pub fn new(socket_addr: SocketAddr, message_receiver: Recipient<WebhookNotification>) -> Self {
        let span = tracing::info_span!("WebhookActor");
        Self {
            socket_addr,
            span,
            message_receiver,
            spawn_handle: None,
        }
    }
}
impl Actor for WebhookActor {
    type Context = actix::Context<Self>;
    fn started(&mut self, ctx: &mut Self::Context) {
        let _gaurd = self.span.enter();
        let address: Recipient<WebhookNotification> = self.message_receiver.clone();
        let server = server::start(self.socket_addr, address);
        let spawn_handle = ctx.spawn(server.in_current_span().into_actor(self));
        self.spawn_handle.replace(spawn_handle);
    }
}
