//
use git_next_core::{git, WebhookId};

use kxio::network;
use secrecy::ExposeSecret as _;

pub async fn unregister(
    webhook_id: &WebhookId,
    repo_details: &git::RepoDetails,
    net: &network::Network,
) -> git::forge::webhook::Result<()> {
    let hostname = &repo_details.forge.hostname();
    let repo_path = &repo_details.repo_path;
    let token = repo_details.forge.token().expose_secret();
    let url = network::NetUrl::new(format!(
        "https://{hostname}/api/v1/repos/{repo_path}/hooks/{webhook_id}?token={token}"
    ));
    let request = network::NetRequest::new(
        network::RequestMethod::Delete,
        url,
        network::NetRequestHeaders::new(),
        network::RequestBody::None,
        network::ResponseType::None,
        None,
        network::NetRequestLogging::None,
    );
    match net.delete(request).await {
        Err(e) => {
            tracing::warn!("Failed to unregister webhook");
            Err(git::forge::webhook::Error::FailedToUnregister(
                e.to_string(),
            ))
        }
        _ => Ok(()),
    }
}
