//
use crate as github;
use git_next_core::{git, server::RepoListenUrl, WebhookId};

use kxio::network;

// https://docs.github.com/en/rest/repos/webhooks?apiVersion=2022-11-28#list-repository-webhooks
pub async fn list(
    github: &github::Github,
    repo_listen_url: &RepoListenUrl,
) -> git::forge::webhook::Result<Vec<WebhookId>> {
    let mut ids: Vec<WebhookId> = vec![];
    let repo_details = &github.repo_details;
    let hostname = &repo_details.forge.hostname();
    let net = &github.net;
    let mut page = 1;
    loop {
        let request = network::NetRequest::new(
            network::RequestMethod::Get,
            network::NetUrl::new(format!(
                "https://api.{hostname}/repos/{}/hooks?page={page}",
                repo_details.repo_path,
            )),
            github::webhook::headers(repo_details.forge.token()),
            network::RequestBody::None,
            network::ResponseType::Json,
            None,
            network::NetRequestLogging::None,
        );
        let result = net.get::<Vec<github::GithubHook>>(request).await;
        match result {
            Ok(response) => {
                let Some(list) = response.response_body() else {
                    #[cfg(not(tarpaulin_include))]
                    // request response is Json so response_body never returns None
                    return Err(git::forge::webhook::Error::NetworkResponseEmpty);
                };
                if list.is_empty() {
                    return Ok(ids);
                }
                for hook in list {
                    if hook
                        .url()
                        .as_ref()
                        .starts_with(&repo_listen_url.to_string())
                    {
                        ids.push(hook.id());
                    }
                }
                page += 1;
            }
            Err(e) => {
                return Err(git::forge::webhook::Error::Network(e));
            }
        };
    }
}
